/* phylips.c - PHYLIPS alignment functions */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdio.h>
#ifdef STDC_HEADERS
# include <string.h>
#endif

#include "align.h"
#include "align/phylip.h"
#include "align/phylips.h"


/* Functions prototypes */
extern int phylipsy_check(FILE *);
extern align_t *phylipsy_parse(FILE *);


/* Check PHYLIPS alignment */
int phylips_check(FILE *f) {
  int i;

  i = phylipsy_check(f);

  return i; }


/* Parse PHYLIPS alignment */
align_t *phylips_parse(FILE *f) {
  align_t *ali;

  ali = phylipsy_parse(f);

  return ali; }


/* Print PHYLIPS alignment */
void phylips_print(FILE *f, align_t *ali) {
  const char *p;
  char **z, **nam, c;
  int i, j;
  size_t n, len;
  sequence_t **s;

  if (ali == NULL) { return; }

  s = ali->seq; len = (*s)->strlen; n = 0;
  while (s && *s) { n++; s++; }

  /* Fix bad sequence names */
  nam = phylip_addfixnames(NULL, ali);

  /* Print alignment header */
  (void)fprintf(f, " %5lu %5lu\n", n, len);

  /* Process all sequences */
  s = ali->seq; z = nam;
  while (s && *s) {
    n = len / 60; if (len % 60) { n++; }
    i = 0;
    while (n--) {
      /* Name */
      p = ""; if (i == 0) { p = *z; }
      (void)fprintf(f, "%-10.10s", p);
      /* Sequence */
      p = (*s)->str + i; j = 60;
      while (j && *p) {
	c = *p;
	if (c == '~' || c == '.') { c = '-'; }
	(void)fputc(c, f);
	j--; p++; }
      (void)fputc('\n', f);
      i += 60; }
    z++; s++; }

  (void)phylip_delfixnames(nam, ali);

  return; }
